@@ -1,15 +1,18 @@
 from collections import abc, deque
+import time
 from urllib.error import URLError
 
+
 from ..core.http.cached_request import CachedRequests
 from ..core.parser.soup import SoupParser
 from ..utils.urls import queryjoin, urljoin
 from ..utils.module import find_function, filter_kwargs
+from ..settings import CRAWL_SUSPENDE_LOOP_POLLING_RATE
 from .exceptions import *
 from .actions import UrlPatternAction, UrlRenderAction
 from .reducer import ReducerMixin
 from .meta import ResponseMeta
-
+from .event import CRAWLING_STARTED, CRAWLING_COMPLETED, VISITING_URL, CRAWLING_STOPPED, CRAWLING_SUSPENDED, catch_crawl_exception
 
 
 
@@ -92,7 +95,7 @@ class BaseCrawler(CachedRequests, SoupParser, ReducerMixin):
     
     def _dispatch_response(self, action, url, headers, context=None):
         if action.payloader:
-            payload = self.dispatch('payloader', action.payloader, context)
+            payload = self.dispatch('payloader', action.payloader, context=context)
             response = self.post(url, payload=payload, headers=headers, **action.__dict__)
         else:
             response = self.get(url, headers=headers, **action.as_kwargs())
@@ -111,11 +114,11 @@ class BaseCrawler(CachedRequests, SoupParser, ReducerMixin):
         return header
     
     
-    def _dispatch_breaker(self, action, response):
+    def _dispatch_breaker(self, action, response, context):
         return self.dispatch(
-            'breaker', action.breaker, response
+            'breaker', action.breaker, response, context=context
         )
-
+    
 
     def get_action(self, name):
         for action in self.urlorders:
@@ -126,18 +129,21 @@ class BaseCrawler(CachedRequests, SoupParser, ReducerMixin):
         return results
 
 
-    def crawl(self, context=None, _response=None, _urlorders=None, _visited=None, _responsemap=None):
+    @catch_crawl_exception
+    def crawl(self, context=None, _response=None, _urlorders=None, _visited=None, _responsemap=None, _visite_count=0, crawl_listener=lambda module, event, context=None: True):
+
+        module_name = f"{self.__class__.__module__}.{self.__class__.__name__}"
         action, *rest = _urlorders or self.urlorders
 
         response_queue = deque([_response])
         _visited = _visited or set()
+        
+        if _response is None:
+            crawl_listener(module_name, CRAWLING_STARTED, {'visite_count': _visite_count})
 
         while response_queue:
             response = response_queue.pop()
 
-            if self._dispatch_breaker(action, response) is True:
-                break
-            
             is_parsable = True
             for link in self._dispatch_renderer(action, response, _responsemap, context):
                 url = self._resolve_link(link, action, response)
@@ -150,6 +156,15 @@ class BaseCrawler(CachedRequests, SoupParser, ReducerMixin):
                 
                 _visited.add(url)
 
+                ## listen visiting url
+                _visite_count += 1
+                crawl_listener(module_name, VISITING_URL, {'visite_count': _visite_count})
+                
+                ## listen breaking loop
+                if self._dispatch_breaker(action, response, context) is True:
+                    crawl_listener(module_name, CRAWLING_STOPPED, {'visite_count': _visite_count})
+                    return
+
                 ## get response
                 header_referer = self._dispatch_referer(action, response)
                 sub_response = self._dispatch_response(action, url, header_referer, context=context)
@@ -179,8 +194,11 @@ class BaseCrawler(CachedRequests, SoupParser, ReducerMixin):
                     if isinstance(action, UrlPatternAction):
                         if action.recursive:
                             response_queue.append(sub_response)
-                    self.crawl(context, sub_response, rest, _visited, meta.responsemap)
+                    self.crawl(context, sub_response, rest, _visited, meta.responsemap, _visite_count, crawl_listener)
 
             # BFO if not passable
             if is_parsable is False:
-                self.crawl(context, response, rest, _visited, meta.responsemap)
+                self.crawl(context, response, rest, _visited, meta.responsemap, _visite_count, crawl_listener)
+        
+        if _response is None:
+            crawl_listener(module_name, CRAWLING_COMPLETED, {'visite_count': _visite_count})
