@@ -4,6 +4,7 @@ from naverkin.kin import test_naverkin
 from navertoon.toon import test_navertoon
 from kofia.crawler import test_kofia
 from druginfo.crawler import test_druginfo
+from costores.crawler import coscores_search
 
 
 def main():
@@ -15,7 +16,7 @@ def main():
     argparser.add_argument('-titleId', '--titleId', default='')
     argparser.add_argument('-start_date', '--start_date', default='')
     argparser.add_argument('-end_date', '--end_date', default='')
-    argparser.add_argument('-search', '--search', nargs='?')
+    argparser.add_argument('-search', '--search', nargs='?', default='')
     argparser.add_argument('-page_range', '--page_range', nargs='?')
 
     args = argparser.parse_args()
@@ -35,6 +36,9 @@ def main():
         test_druginfo(
             q=args.search
         )
+    
+    if args.appname == 'coscores':
+        coscores_search(search=args.search)
         
 
 
